# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import (
    permission_policy_list_params,
    permission_policy_create_params,
    permission_policy_update_params,
    permission_policy_add_to_user_params,
    permission_policy_remove_from_user_params,
    permission_policy_add_to_user_group_params,
    permission_policy_add_to_service_user_params,
    permission_policy_add_to_support_user_params,
    permission_policy_remove_from_user_group_params,
    permission_policy_remove_from_service_user_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.permission_policy_response import PermissionPolicyResponse
from ..types.permission_statement_response_param import PermissionStatementResponseParam
from ..types.permission_policy_add_to_user_response import PermissionPolicyAddToUserResponse
from ..types.permission_policy_remove_from_user_response import PermissionPolicyRemoveFromUserResponse
from ..types.permission_policy_add_to_user_group_response import PermissionPolicyAddToUserGroupResponse
from ..types.permission_policy_add_to_service_user_response import PermissionPolicyAddToServiceUserResponse
from ..types.permission_policy_add_to_support_user_response import PermissionPolicyAddToSupportUserResponse
from ..types.permission_policy_remove_from_user_group_response import PermissionPolicyRemoveFromUserGroupResponse
from ..types.permission_policy_remove_from_service_user_response import PermissionPolicyRemoveFromServiceUserResponse
from ..types.permission_policy_remove_from_support_user_response import PermissionPolicyRemoveFromSupportUserResponse

__all__ = ["PermissionPoliciesResource", "AsyncPermissionPoliciesResource"]


class PermissionPoliciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PermissionPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return PermissionPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PermissionPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return PermissionPoliciesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        permission_policy: Iterable[PermissionStatementResponseParam],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Create a new Permission Policy

        **NOTE:** When you set up a policy statement for this call using the
        `permissionPolicy` request parameter to specify the `effect`, `action`, and
        `resource`, you must use all lower case and the format as shown in this example
        for a Permission Policy statement that grants full CRUD access to all meters:

        ```
        "permissionPolicy" : [
                {
                        "effect" : "allow",
                        "action" : [
                        "config:create",
                        "config:delete",
                        "config:retrieve",
                        "config:update"
                        ]
                        "resource" : [
                        "config:meter/*"
                        ]
                }
        ]
        ```

        For more details and further examples, see
        [Understanding, Creating, and Managing Permission Policies](https://www.m3ter.com/docs/guides/organization-and-access-management/creating-and-managing-permissions#permission-policy-statements---available-actions-and-resources)
        in our main Documentation.

        Args:
          name

          permission_policy

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - do not use
                for Create. On initial Create, version is set at 1 and listed in the response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/permissionpolicies",
            body=maybe_transform(
                {
                    "name": name,
                    "permission_policy": permission_policy,
                    "version": version,
                },
                permission_policy_create_params.PermissionPolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Retrieve the permission policy for the UUID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        permission_policy: Iterable[PermissionStatementResponseParam],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Update a Permission Policy for the UUID

        **NOTE:** When you set up a policy statement for this call to specify the
        `effect`, `action`, and `resource`, you must use all lower case and the format
        as shown in this example - a Permission Policy statement that grants full CRUD
        access to all meters:

        ```
        "permissionPolicy" : [
                {
                        "effect" : "allow",
                        "action" : [
                        "config:create",
                        "config:delete",
                        "config:retrieve",
                        "config:update"
                        ]
                        "resource" : [
                        "config:meter/*"
                        ]
                }
        ]
        ```

        For more details and further examples, see
        [Understanding, Creating, and Managing Permission Policies](https://www.m3ter.com/docs/guides/organization-and-access-management/creating-and-managing-permissions#permission-policy-statements---available-actions-and-resources)
        in our main Documentation.

        Args:
          name

          permission_policy

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - do not use
                for Create. On initial Create, version is set at 1 and listed in the response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            body=maybe_transform(
                {
                    "name": name,
                    "permission_policy": permission_policy,
                    "version": version,
                },
                permission_policy_update_params.PermissionPolicyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[PermissionPolicyResponse]:
        """
        Retrieve a list of PermissionPolicy entities

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of permission polices to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/permissionpolicies",
            page=SyncCursor[PermissionPolicyResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    permission_policy_list_params.PermissionPolicyListParams,
                ),
            ),
            model=PermissionPolicyResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Delete the PermissionPolicy for the UUID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    def add_to_service_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToServiceUserResponse:
        """
        Add a permission policy to a service user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtoserviceuser",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_service_user_params.PermissionPolicyAddToServiceUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToServiceUserResponse,
        )

    def add_to_support_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToSupportUserResponse:
        """
        Add a permission policy to support users for an organization.

        Args:
          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtosupportusers",
            body=maybe_transform(
                {"version": version},
                permission_policy_add_to_support_user_params.PermissionPolicyAddToSupportUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToSupportUserResponse,
        )

    def add_to_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToUserResponse:
        """
        Add a permission policy to a user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtouser",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_user_params.PermissionPolicyAddToUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToUserResponse,
        )

    def add_to_user_group(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToUserGroupResponse:
        """
        Add a permission Policy to a user group

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtousergroup",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_user_group_params.PermissionPolicyAddToUserGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToUserGroupResponse,
        )

    def remove_from_service_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromServiceUserResponse:
        """
        Remove a permission policy from a service user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromserviceuser",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_service_user_params.PermissionPolicyRemoveFromServiceUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromServiceUserResponse,
        )

    def remove_from_support_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromSupportUserResponse:
        """
        Remove a permission policy from support users for an organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromsupportusers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromSupportUserResponse,
        )

    def remove_from_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromUserResponse:
        """
        Remove a permission policy from a user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromuser",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_user_params.PermissionPolicyRemoveFromUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromUserResponse,
        )

    def remove_from_user_group(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromUserGroupResponse:
        """
        Remove a permission policy from a user group.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromusergroup",
            body=maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_user_group_params.PermissionPolicyRemoveFromUserGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromUserGroupResponse,
        )


class AsyncPermissionPoliciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPermissionPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPermissionPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPermissionPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncPermissionPoliciesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        name: str,
        permission_policy: Iterable[PermissionStatementResponseParam],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Create a new Permission Policy

        **NOTE:** When you set up a policy statement for this call using the
        `permissionPolicy` request parameter to specify the `effect`, `action`, and
        `resource`, you must use all lower case and the format as shown in this example
        for a Permission Policy statement that grants full CRUD access to all meters:

        ```
        "permissionPolicy" : [
                {
                        "effect" : "allow",
                        "action" : [
                        "config:create",
                        "config:delete",
                        "config:retrieve",
                        "config:update"
                        ]
                        "resource" : [
                        "config:meter/*"
                        ]
                }
        ]
        ```

        For more details and further examples, see
        [Understanding, Creating, and Managing Permission Policies](https://www.m3ter.com/docs/guides/organization-and-access-management/creating-and-managing-permissions#permission-policy-statements---available-actions-and-resources)
        in our main Documentation.

        Args:
          name

          permission_policy

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - do not use
                for Create. On initial Create, version is set at 1 and listed in the response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "permission_policy": permission_policy,
                    "version": version,
                },
                permission_policy_create_params.PermissionPolicyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Retrieve the permission policy for the UUID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        name: str,
        permission_policy: Iterable[PermissionStatementResponseParam],
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Update a Permission Policy for the UUID

        **NOTE:** When you set up a policy statement for this call to specify the
        `effect`, `action`, and `resource`, you must use all lower case and the format
        as shown in this example - a Permission Policy statement that grants full CRUD
        access to all meters:

        ```
        "permissionPolicy" : [
                {
                        "effect" : "allow",
                        "action" : [
                        "config:create",
                        "config:delete",
                        "config:retrieve",
                        "config:update"
                        ]
                        "resource" : [
                        "config:meter/*"
                        ]
                }
        ]
        ```

        For more details and further examples, see
        [Understanding, Creating, and Managing Permission Policies](https://www.m3ter.com/docs/guides/organization-and-access-management/creating-and-managing-permissions#permission-policy-statements---available-actions-and-resources)
        in our main Documentation.

        Args:
          name

          permission_policy

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - do not use
                for Create. On initial Create, version is set at 1 and listed in the response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "permission_policy": permission_policy,
                    "version": version,
                },
                permission_policy_update_params.PermissionPolicyUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PermissionPolicyResponse, AsyncCursor[PermissionPolicyResponse]]:
        """
        Retrieve a list of PermissionPolicy entities

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of permission polices to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/permissionpolicies",
            page=AsyncCursor[PermissionPolicyResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    permission_policy_list_params.PermissionPolicyListParams,
                ),
            ),
            model=PermissionPolicyResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyResponse:
        """
        Delete the PermissionPolicy for the UUID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/permissionpolicies/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyResponse,
        )

    async def add_to_service_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToServiceUserResponse:
        """
        Add a permission policy to a service user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtoserviceuser",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_service_user_params.PermissionPolicyAddToServiceUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToServiceUserResponse,
        )

    async def add_to_support_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToSupportUserResponse:
        """
        Add a permission policy to support users for an organization.

        Args:
          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtosupportusers",
            body=await async_maybe_transform(
                {"version": version},
                permission_policy_add_to_support_user_params.PermissionPolicyAddToSupportUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToSupportUserResponse,
        )

    async def add_to_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToUserResponse:
        """
        Add a permission policy to a user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtouser",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_user_params.PermissionPolicyAddToUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToUserResponse,
        )

    async def add_to_user_group(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyAddToUserGroupResponse:
        """
        Add a permission Policy to a user group

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/addtousergroup",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_add_to_user_group_params.PermissionPolicyAddToUserGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyAddToUserGroupResponse,
        )

    async def remove_from_service_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromServiceUserResponse:
        """
        Remove a permission policy from a service user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromserviceuser",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_service_user_params.PermissionPolicyRemoveFromServiceUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromServiceUserResponse,
        )

    async def remove_from_support_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromSupportUserResponse:
        """
        Remove a permission policy from support users for an organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromsupportusers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromSupportUserResponse,
        )

    async def remove_from_user(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromUserResponse:
        """
        Remove a permission policy from a user.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromuser",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_user_params.PermissionPolicyRemoveFromUserParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromUserResponse,
        )

    async def remove_from_user_group(
        self,
        permission_policy_id: str,
        *,
        org_id: str | None = None,
        principal_id: str,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PermissionPolicyRemoveFromUserGroupResponse:
        """
        Remove a permission policy from a user group.

        Args:
          principal_id

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not permission_policy_id:
            raise ValueError(
                f"Expected a non-empty value for `permission_policy_id` but received {permission_policy_id!r}"
            )
        return await self._post(
            f"/organizations/{org_id}/permissionpolicies/{permission_policy_id}/removefromusergroup",
            body=await async_maybe_transform(
                {
                    "principal_id": principal_id,
                    "version": version,
                },
                permission_policy_remove_from_user_group_params.PermissionPolicyRemoveFromUserGroupParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PermissionPolicyRemoveFromUserGroupResponse,
        )


class PermissionPoliciesResourceWithRawResponse:
    def __init__(self, permission_policies: PermissionPoliciesResource) -> None:
        self._permission_policies = permission_policies

        self.create = to_raw_response_wrapper(
            permission_policies.create,
        )
        self.retrieve = to_raw_response_wrapper(
            permission_policies.retrieve,
        )
        self.update = to_raw_response_wrapper(
            permission_policies.update,
        )
        self.list = to_raw_response_wrapper(
            permission_policies.list,
        )
        self.delete = to_raw_response_wrapper(
            permission_policies.delete,
        )
        self.add_to_service_user = to_raw_response_wrapper(
            permission_policies.add_to_service_user,
        )
        self.add_to_support_user = to_raw_response_wrapper(
            permission_policies.add_to_support_user,
        )
        self.add_to_user = to_raw_response_wrapper(
            permission_policies.add_to_user,
        )
        self.add_to_user_group = to_raw_response_wrapper(
            permission_policies.add_to_user_group,
        )
        self.remove_from_service_user = to_raw_response_wrapper(
            permission_policies.remove_from_service_user,
        )
        self.remove_from_support_user = to_raw_response_wrapper(
            permission_policies.remove_from_support_user,
        )
        self.remove_from_user = to_raw_response_wrapper(
            permission_policies.remove_from_user,
        )
        self.remove_from_user_group = to_raw_response_wrapper(
            permission_policies.remove_from_user_group,
        )


class AsyncPermissionPoliciesResourceWithRawResponse:
    def __init__(self, permission_policies: AsyncPermissionPoliciesResource) -> None:
        self._permission_policies = permission_policies

        self.create = async_to_raw_response_wrapper(
            permission_policies.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            permission_policies.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            permission_policies.update,
        )
        self.list = async_to_raw_response_wrapper(
            permission_policies.list,
        )
        self.delete = async_to_raw_response_wrapper(
            permission_policies.delete,
        )
        self.add_to_service_user = async_to_raw_response_wrapper(
            permission_policies.add_to_service_user,
        )
        self.add_to_support_user = async_to_raw_response_wrapper(
            permission_policies.add_to_support_user,
        )
        self.add_to_user = async_to_raw_response_wrapper(
            permission_policies.add_to_user,
        )
        self.add_to_user_group = async_to_raw_response_wrapper(
            permission_policies.add_to_user_group,
        )
        self.remove_from_service_user = async_to_raw_response_wrapper(
            permission_policies.remove_from_service_user,
        )
        self.remove_from_support_user = async_to_raw_response_wrapper(
            permission_policies.remove_from_support_user,
        )
        self.remove_from_user = async_to_raw_response_wrapper(
            permission_policies.remove_from_user,
        )
        self.remove_from_user_group = async_to_raw_response_wrapper(
            permission_policies.remove_from_user_group,
        )


class PermissionPoliciesResourceWithStreamingResponse:
    def __init__(self, permission_policies: PermissionPoliciesResource) -> None:
        self._permission_policies = permission_policies

        self.create = to_streamed_response_wrapper(
            permission_policies.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            permission_policies.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            permission_policies.update,
        )
        self.list = to_streamed_response_wrapper(
            permission_policies.list,
        )
        self.delete = to_streamed_response_wrapper(
            permission_policies.delete,
        )
        self.add_to_service_user = to_streamed_response_wrapper(
            permission_policies.add_to_service_user,
        )
        self.add_to_support_user = to_streamed_response_wrapper(
            permission_policies.add_to_support_user,
        )
        self.add_to_user = to_streamed_response_wrapper(
            permission_policies.add_to_user,
        )
        self.add_to_user_group = to_streamed_response_wrapper(
            permission_policies.add_to_user_group,
        )
        self.remove_from_service_user = to_streamed_response_wrapper(
            permission_policies.remove_from_service_user,
        )
        self.remove_from_support_user = to_streamed_response_wrapper(
            permission_policies.remove_from_support_user,
        )
        self.remove_from_user = to_streamed_response_wrapper(
            permission_policies.remove_from_user,
        )
        self.remove_from_user_group = to_streamed_response_wrapper(
            permission_policies.remove_from_user_group,
        )


class AsyncPermissionPoliciesResourceWithStreamingResponse:
    def __init__(self, permission_policies: AsyncPermissionPoliciesResource) -> None:
        self._permission_policies = permission_policies

        self.create = async_to_streamed_response_wrapper(
            permission_policies.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            permission_policies.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            permission_policies.update,
        )
        self.list = async_to_streamed_response_wrapper(
            permission_policies.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            permission_policies.delete,
        )
        self.add_to_service_user = async_to_streamed_response_wrapper(
            permission_policies.add_to_service_user,
        )
        self.add_to_support_user = async_to_streamed_response_wrapper(
            permission_policies.add_to_support_user,
        )
        self.add_to_user = async_to_streamed_response_wrapper(
            permission_policies.add_to_user,
        )
        self.add_to_user_group = async_to_streamed_response_wrapper(
            permission_policies.add_to_user_group,
        )
        self.remove_from_service_user = async_to_streamed_response_wrapper(
            permission_policies.remove_from_service_user,
        )
        self.remove_from_support_user = async_to_streamed_response_wrapper(
            permission_policies.remove_from_support_user,
        )
        self.remove_from_user = async_to_streamed_response_wrapper(
            permission_policies.remove_from_user,
        )
        self.remove_from_user_group = async_to_streamed_response_wrapper(
            permission_policies.remove_from_user_group,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["PlanResponse"]


class PlanResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    account_id: Optional[str] = FieldInfo(alias="accountId", default=None)
    """_(Optional)_.

    The Account ID for which this plan was created as custom/bespoke. A
    custom/bespoke Plan can only be attached to the specified Account.
    """

    bespoke: Optional[bool] = None
    """
    TRUE/FALSE flag indicating whether the plan is custom/bespoke for a particular
    Account.
    """

    code: Optional[str] = None
    """Unique short code reference for the Plan."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The id of the user who created this plan."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime _(in ISO-8601 format)_ when the plan was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime _(in ISO-8601 format)_ when the plan was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The id of the user who last modified this plan."""

    minimum_spend: Optional[float] = FieldInfo(alias="minimumSpend", default=None)
    """
    The product minimum spend amount per billing cycle for end customer Accounts on
    a priced Plan.

    _(Optional)_. Overrides PlanTemplate value.
    """

    minimum_spend_accounting_product_id: Optional[str] = FieldInfo(
        alias="minimumSpendAccountingProductId", default=None
    )
    """
    Optional Product ID this plan's minimum spend should be attributed to for
    accounting purposes
    """

    minimum_spend_bill_in_advance: Optional[bool] = FieldInfo(alias="minimumSpendBillInAdvance", default=None)
    """When TRUE, minimum spend is billed at the start of each billing period.

    When FALSE, minimum spend is billed at the end of each billing period.

    _(Optional)_. Overrides the setting at PlanTemplate level for minimum spend
    billing in arrears/in advance.
    """

    minimum_spend_description: Optional[str] = FieldInfo(alias="minimumSpendDescription", default=None)
    """Minimum spend description _(displayed on the bill line item)_."""

    name: Optional[str] = None
    """Descriptive name for the Plan."""

    ordinal: Optional[int] = None
    """
    Assigns a rank or position to the Plan in your order of pricing plans - lower
    numbers represent more basic pricing plans; higher numbers represent more
    premium pricing plans.

    _(Optional)_. Overrides PlanTemplate value.

    **NOTE:** **DEPRECATED** - no longer used.
    """

    plan_template_id: Optional[str] = FieldInfo(alias="planTemplateId", default=None)
    """UUID of the PlanTemplate the Plan belongs to."""

    product_id: Optional[str] = FieldInfo(alias="productId", default=None)
    """UUID of the Product the Plan belongs to."""

    standing_charge: Optional[float] = FieldInfo(alias="standingCharge", default=None)
    """The standing charge applied to bills for end customers. This is prorated.

    _(Optional)_. Overrides PlanTemplate value.
    """

    standing_charge_accounting_product_id: Optional[str] = FieldInfo(
        alias="standingChargeAccountingProductId", default=None
    )
    """
    Optional Product ID this plan's standing charge should be attributed to for
    accounting purposes
    """

    standing_charge_bill_in_advance: Optional[bool] = FieldInfo(alias="standingChargeBillInAdvance", default=None)
    """When TRUE, standing charge is billed at the start of each billing period.

    When FALSE, standing charge is billed at the end of each billing period.

    _(Optional)_. Overrides the setting at PlanTemplate level for standing charge
    billing in arrears/in advance.
    """

    standing_charge_description: Optional[str] = FieldInfo(alias="standingChargeDescription", default=None)
    """Standing charge description _(displayed on the bill line item)_."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .balances import (
    BalancesResource,
    AsyncBalancesResource,
    BalancesResourceWithRawResponse,
    AsyncBalancesResourceWithRawResponse,
    BalancesResourceWithStreamingResponse,
    AsyncBalancesResourceWithStreamingResponse,
)
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)

__all__ = [
    "TransactionsResource",
    "AsyncTransactionsResource",
    "TransactionsResourceWithRawResponse",
    "AsyncTransactionsResourceWithRawResponse",
    "TransactionsResourceWithStreamingResponse",
    "AsyncTransactionsResourceWithStreamingResponse",
    "BalancesResource",
    "AsyncBalancesResource",
    "BalancesResourceWithRawResponse",
    "AsyncBalancesResourceWithRawResponse",
    "BalancesResourceWithStreamingResponse",
    "AsyncBalancesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AccountPlanListParams"]


class AccountPlanListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account: str
    """
    The unique identifier (UUID) for the Account whose AccountPlans and
    AccountPlanGroups you want to retrieve.
    """

    contract: Optional[str]

    date: str
    """
    The specific date for which you want to retrieve active AccountPlans and
    AccountPlanGroups.
    """

    ids: List[str]
    """
    A list of unique identifiers (UUIDs) for specific AccountPlans and
    AccountPlanGroups you want to retrieve.
    """

    includeall: bool
    """
    A Boolean flag that specifies whether to include both active and inactive
    AccountPlans and AccountPlanGroups in the list.

    - **TRUE** - both active and inactive AccountPlans and AccountPlanGroups are
      included in the list.
    - **FALSE** - only active AccountPlans and AccountPlanGroups are retrieved in
      the list.
    """

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """
    The `nextToken` for retrieving the next page of AccountPlans and
    AccountPlanGroups. It is used to fetch the next page of AccountPlans and
    AccountPlanGroups in a paginated list.
    """

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """The maximum number of AccountPlans and AccountPlanGroups to return per page."""

    plan: str
    """
    The unique identifier (UUID) for the Plan or Plan Group whose associated
    AccountPlans or AccountPlanGroups you want to retrieve.
    """

    product: str
    """
    The unique identifier (UUID) for the Product whose associated AccountPlans you
    want to retrieve.

    **NOTE:** You cannot use the `product` query parameter as a single filter
    condition, but must always use it in combination with the `account` query
    parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared_params.currency_conversion import CurrencyConversion

__all__ = ["BillJobCreateParams"]


class BillJobCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_ids: Annotated[List[str], PropertyInfo(alias="accountIds")]
    """
    An array of UUIDs representing the end customer Accounts associated with the
    BillJob.
    """

    bill_date: Annotated[Union[str, date], PropertyInfo(alias="billDate", format="iso8601")]
    """
    The specific billing date _(in ISO 8601 format)_, determining when the Bill was
    generated.

    For example: `"2023-01-24"`.
    """

    bill_frequency_interval: Annotated[int, PropertyInfo(alias="billFrequencyInterval")]
    """How often Bills are issued - used in conjunction with `billingFrequency`.

    For example, if `billingFrequency` is set to Monthly and `billFrequencyInterval`
    is set to 3, Bills are issued every three months.
    """

    billing_frequency: Annotated[
        Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY", "AD_HOC"], PropertyInfo(alias="billingFrequency")
    ]
    """How often Bills are generated.

    - **Daily**. Starting at midnight each day, covering a twenty-four hour period
      following.

    - **Weekly**. Starting at midnight on a Monday morning covering the seven-day
      period following.

    - **Monthly**. Starting at midnight on the morning of the first day of each
      month covering the entire calendar month following.

    - **Annually**. Starting at midnight on the morning of the first day of each
      year covering the entire calendar year following.

    - **Ad_Hoc**. Use this setting when a custom billing schedule is used for
      billing an Account, such as for billing of Prepayment/Commitment fees using a
      custom billing schedule.
    """

    currency_conversions: Annotated[Iterable[CurrencyConversion], PropertyInfo(alias="currencyConversions")]
    """
    An array of currency conversion rates from Bill currency to Organization
    currency. For example, if Account is billed in GBP and Organization is set to
    USD, Bill line items are calculated in GBP and then converted to USD using the
    defined rate.
    """

    day_epoch: Annotated[Union[str, date], PropertyInfo(alias="dayEpoch", format="iso8601")]
    """
    The starting date _(epoch)_ for Daily billing frequency _(in ISO 8601 format)_,
    determining the first Bill date for daily Bills.
    """

    due_date: Annotated[Union[str, date], PropertyInfo(alias="dueDate", format="iso8601")]
    """The due date _(in ISO 8601 format)_ for payment of the Bill.

    For example: `"2023-02-24"`.
    """

    external_invoice_date: Annotated[Union[str, date], PropertyInfo(alias="externalInvoiceDate", format="iso8601")]
    """
    For accounting purposes, the date set at Organization level to use for external
    invoicing with respect to billing periods - two options:

    - `FIRST_DAY_OF_NEXT_PERIOD` _(Default)_. Used when you want to recognize usage
      revenue in the following period.
    - `LAST_DAY_OF_ARREARS`. Used when you want to recognize usage revenue in the
      same period that it's consumed, instead of in the following period.

    For example, if the retrieved Bill was on a monthly billing frequency and the
    billing period for the Bill is September 2023 and the _External invoice date_ is
    set at `FIRST_DAY_OF_NEXT_PERIOD`, then the `externalInvoiceDate` will be
    `"2023-10-01"`.

    **NOTE:** To change the `externalInvoiceDate` setting for your Organization, you
    can use the
    [Update OrganizationConfig](https://www.m3ter.com/docs/api#tag/OrganizationConfig/operation/GetOrganizationConfig)
    call.
    """

    last_date_in_billing_period: Annotated[
        Union[str, date], PropertyInfo(alias="lastDateInBillingPeriod", format="iso8601")
    ]
    """
    Specifies the date _(in ISO 8601 format)_ of the last day in the billing period,
    defining the time range for the associated Bills.

    For example: `"2023-03-24"`.
    """

    month_epoch: Annotated[Union[str, date], PropertyInfo(alias="monthEpoch", format="iso8601")]
    """
    The starting date _(epoch)_ for Monthly billing frequency _(in ISO 8601
    format)_, determining the first Bill date for monthly Bills.
    """

    target_currency: Annotated[str, PropertyInfo(alias="targetCurrency")]
    """The currency code used for the Bill, such as USD, GBP, or EUR."""

    timezone: str
    """
    Specifies the time zone used for the generated Bills, ensuring alignment with
    the local time zone.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """

    week_epoch: Annotated[Union[str, date], PropertyInfo(alias="weekEpoch", format="iso8601")]
    """
    The starting date _(epoch)_ for Weekly billing frequency _(in ISO 8601 format)_,
    determining the first Bill date for weekly Bills.
    """

    year_epoch: Annotated[Union[str, date], PropertyInfo(alias="yearEpoch", format="iso8601")]
    """
    The starting date _(epoch)_ for Yearly billing frequency _(in ISO 8601 format)_,
    determining the first Bill date for yearly Bills.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ProductResponse"]


class ProductResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    code: Optional[str] = None
    """A unique short code to identify the Product.

    It should not contain control chracters or spaces.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The unique identifier (UUID) of the user who created this Product."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The date and time _(in ISO-8601 format)_ when the Product was created."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The date and time _(in ISO-8601 format)_ when the Product was last modified."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The unique identifier (UUID) of the user who last modified this Product."""

    name: Optional[str] = None
    """Descriptive name for the Product providing context and information."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import (
    scheduled_event_configuration_list_params,
    scheduled_event_configuration_create_params,
    scheduled_event_configuration_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.scheduled_event_configuration_response import ScheduledEventConfigurationResponse

__all__ = ["ScheduledEventConfigurationsResource", "AsyncScheduledEventConfigurationsResource"]


class ScheduledEventConfigurationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ScheduledEventConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ScheduledEventConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScheduledEventConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ScheduledEventConfigurationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        entity: str,
        field: str,
        name: str,
        offset: int,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Create a new ScheduledEventConfiguration.

        Args:
          entity: The referenced configuration or billing entity for which the desired scheduled
              Event will trigger.

          field: A DateTime field for which the desired scheduled Event will trigger - this must
              be a DateTime field on the referenced billing or configuration entity.

          name: The name of the custom Scheduled Event Configuration.

              This must be in the format:

              - scheduled._name of entity_._custom event name_

              For example:

              - `scheduled.bill.endDateEvent`

          offset: The offset in days from the specified DateTime field on the referenced entity
              when the scheduled Event will trigger.

          version:
              The version number of the scheduled event configuration:

              - **Create entity**: Not valid for initial insertion - do not use for Create. On
                initial Create, version is set at 1 and listed in the response.
              - **Update Entity**: On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/scheduledevents/configurations",
            body=maybe_transform(
                {
                    "entity": entity,
                    "field": field,
                    "name": name,
                    "offset": offset,
                    "version": version,
                },
                scheduled_event_configuration_create_params.ScheduledEventConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Retrieve a ScheduledEventConfiguration for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        entity: str,
        field: str,
        name: str,
        offset: int,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Update a ScheduledEventConfiguration for the given UUID.

        Args:
          entity: The referenced configuration or billing entity for which the desired scheduled
              Event will trigger.

          field: A DateTime field for which the desired scheduled Event will trigger - this must
              be a DateTime field on the referenced billing or configuration entity.

          name: The name of the custom Scheduled Event Configuration.

              This must be in the format:

              - scheduled._name of entity_._custom event name_

              For example:

              - `scheduled.bill.endDateEvent`

          offset: The offset in days from the specified DateTime field on the referenced entity
              when the scheduled Event will trigger.

          version:
              The version number of the scheduled event configuration:

              - **Create entity**: Not valid for initial insertion - do not use for Create. On
                initial Create, version is set at 1 and listed in the response.
              - **Update Entity**: On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            body=maybe_transform(
                {
                    "entity": entity,
                    "field": field,
                    "name": name,
                    "offset": offset,
                    "version": version,
                },
                scheduled_event_configuration_update_params.ScheduledEventConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[ScheduledEventConfigurationResponse]:
        """
        Retrieve a list of ScheduledEventConfiguration entities

        Args:
          ids: list of UUIDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of ScheduledEventConfigurations to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/scheduledevents/configurations",
            page=SyncCursor[ScheduledEventConfigurationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    scheduled_event_configuration_list_params.ScheduledEventConfigurationListParams,
                ),
            ),
            model=ScheduledEventConfigurationResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Delete the ScheduledEventConfiguration for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )


class AsyncScheduledEventConfigurationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncScheduledEventConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScheduledEventConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScheduledEventConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncScheduledEventConfigurationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        entity: str,
        field: str,
        name: str,
        offset: int,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Create a new ScheduledEventConfiguration.

        Args:
          entity: The referenced configuration or billing entity for which the desired scheduled
              Event will trigger.

          field: A DateTime field for which the desired scheduled Event will trigger - this must
              be a DateTime field on the referenced billing or configuration entity.

          name: The name of the custom Scheduled Event Configuration.

              This must be in the format:

              - scheduled._name of entity_._custom event name_

              For example:

              - `scheduled.bill.endDateEvent`

          offset: The offset in days from the specified DateTime field on the referenced entity
              when the scheduled Event will trigger.

          version:
              The version number of the scheduled event configuration:

              - **Create entity**: Not valid for initial insertion - do not use for Create. On
                initial Create, version is set at 1 and listed in the response.
              - **Update Entity**: On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/scheduledevents/configurations",
            body=await async_maybe_transform(
                {
                    "entity": entity,
                    "field": field,
                    "name": name,
                    "offset": offset,
                    "version": version,
                },
                scheduled_event_configuration_create_params.ScheduledEventConfigurationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Retrieve a ScheduledEventConfiguration for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        entity: str,
        field: str,
        name: str,
        offset: int,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Update a ScheduledEventConfiguration for the given UUID.

        Args:
          entity: The referenced configuration or billing entity for which the desired scheduled
              Event will trigger.

          field: A DateTime field for which the desired scheduled Event will trigger - this must
              be a DateTime field on the referenced billing or configuration entity.

          name: The name of the custom Scheduled Event Configuration.

              This must be in the format:

              - scheduled._name of entity_._custom event name_

              For example:

              - `scheduled.bill.endDateEvent`

          offset: The offset in days from the specified DateTime field on the referenced entity
              when the scheduled Event will trigger.

          version:
              The version number of the scheduled event configuration:

              - **Create entity**: Not valid for initial insertion - do not use for Create. On
                initial Create, version is set at 1 and listed in the response.
              - **Update Entity**: On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            body=await async_maybe_transform(
                {
                    "entity": entity,
                    "field": field,
                    "name": name,
                    "offset": offset,
                    "version": version,
                },
                scheduled_event_configuration_update_params.ScheduledEventConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ScheduledEventConfigurationResponse, AsyncCursor[ScheduledEventConfigurationResponse]]:
        """
        Retrieve a list of ScheduledEventConfiguration entities

        Args:
          ids: list of UUIDs to retrieve

          next_token: nextToken for multi page retrievals

          page_size: Number of ScheduledEventConfigurations to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/scheduledevents/configurations",
            page=AsyncCursor[ScheduledEventConfigurationResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    scheduled_event_configuration_list_params.ScheduledEventConfigurationListParams,
                ),
            ),
            model=ScheduledEventConfigurationResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ScheduledEventConfigurationResponse:
        """
        Delete the ScheduledEventConfiguration for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/scheduledevents/configurations/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ScheduledEventConfigurationResponse,
        )


class ScheduledEventConfigurationsResourceWithRawResponse:
    def __init__(self, scheduled_event_configurations: ScheduledEventConfigurationsResource) -> None:
        self._scheduled_event_configurations = scheduled_event_configurations

        self.create = to_raw_response_wrapper(
            scheduled_event_configurations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            scheduled_event_configurations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            scheduled_event_configurations.update,
        )
        self.list = to_raw_response_wrapper(
            scheduled_event_configurations.list,
        )
        self.delete = to_raw_response_wrapper(
            scheduled_event_configurations.delete,
        )


class AsyncScheduledEventConfigurationsResourceWithRawResponse:
    def __init__(self, scheduled_event_configurations: AsyncScheduledEventConfigurationsResource) -> None:
        self._scheduled_event_configurations = scheduled_event_configurations

        self.create = async_to_raw_response_wrapper(
            scheduled_event_configurations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            scheduled_event_configurations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            scheduled_event_configurations.update,
        )
        self.list = async_to_raw_response_wrapper(
            scheduled_event_configurations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            scheduled_event_configurations.delete,
        )


class ScheduledEventConfigurationsResourceWithStreamingResponse:
    def __init__(self, scheduled_event_configurations: ScheduledEventConfigurationsResource) -> None:
        self._scheduled_event_configurations = scheduled_event_configurations

        self.create = to_streamed_response_wrapper(
            scheduled_event_configurations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            scheduled_event_configurations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            scheduled_event_configurations.update,
        )
        self.list = to_streamed_response_wrapper(
            scheduled_event_configurations.list,
        )
        self.delete = to_streamed_response_wrapper(
            scheduled_event_configurations.delete,
        )


class AsyncScheduledEventConfigurationsResourceWithStreamingResponse:
    def __init__(self, scheduled_event_configurations: AsyncScheduledEventConfigurationsResource) -> None:
        self._scheduled_event_configurations = scheduled_event_configurations

        self.create = async_to_streamed_response_wrapper(
            scheduled_event_configurations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            scheduled_event_configurations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            scheduled_event_configurations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            scheduled_event_configurations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            scheduled_event_configurations.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AuthenticationGetBearerTokenParams"]


class AuthenticationGetBearerTokenParams(TypedDict, total=False):
    grant_type: Required[Literal["client_credentials"]]
    """The grant type."""

    scope: str
    """Not used. The JWT scope."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["IntegrationConfigurationGetByEntityParams"]


class IntegrationConfigurationGetByEntityParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    entity_id: Annotated[str, PropertyInfo(alias="entityId")]
    """UUID of the entity to retrieve IntegrationConfigs for"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .address import Address
from .._models import BaseModel

__all__ = ["AccountResponse"]


class AccountResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    version: int
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    address: Optional[Address] = None
    """Contact address."""

    auto_generate_statement_mode: Optional[Literal["NONE", "JSON", "JSON_AND_CSV"]] = FieldInfo(
        alias="autoGenerateStatementMode", default=None
    )
    """Specify whether to auto-generate statements once Bills are approved or locked.

    - **None**. Statements will not be auto-generated.
    - **JSON**. Statements are auto-generated in JSON format.
    - **JSON and CSV**. Statements are auto-generated in both JSON and CSV formats.
    """

    bill_epoch: Optional[date] = FieldInfo(alias="billEpoch", default=None)
    """Defines first bill date for Account Bills.

    For example, if the Plan attached to the Account is set for monthly billing
    frequency and you set the first bill date to be January 1st, Bills are created
    every month starting on that date.

    Optional attribute - if not defined, then first bill date is determined by the
    Epoch settings at Organizational level.
    """

    code: Optional[str] = None
    """Code of the Account. This is a unique short code used for the Account."""

    config_data: Optional[Dict[str, object]] = FieldInfo(alias="configData", default=None)
    """Configuration data for the Account"""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The ID of the user who created the account."""

    credit_application_order: Optional[List[Literal["PREPAYMENT", "BALANCE"]]] = FieldInfo(
        alias="creditApplicationOrder", default=None
    )
    """
    The order in which any Prepayment or Balance amounts on the Account are to be
    drawn-down against for billing. Four options:

    - `"PREPAYMENT","BALANCE"`. Draw-down against Prepayment credit before Balance
      credit.
    - `"BALANCE","PREPAYMENT"`. Draw-down against Balance credit before Prepayment
      credit.
    - `"PREPAYMENT"`. Only draw-down against Prepayment credit.
    - `"BALANCE"`. Only draw-down against Balance credit.
    """

    currency: Optional[str] = None
    """Account level billing currency, such as USD or GBP. Optional attribute:

    - If you define an Account currency, this will be used for bills.
    - If you do not define a currency, the billing currency defined at
      Organizational will be used.

    **Note:** If you've attached a Plan to the Account that uses a different
    currency to the billing currency, then you must add the relevant currency
    conversion rate at Organization level to ensure the billing process can convert
    line items calculated using the Plan currency into the selected billing
    currency. If you don't add these conversion rates, then bills will fail for the
    Account.
    """

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    days_before_bill_due: Optional[int] = FieldInfo(alias="daysBeforeBillDue", default=None)
    """
    The number of days after the Bill generation date shown on Bills as the due
    date.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """The DateTime when the Account was created _(in ISO 8601 format)_."""

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """The DateTime when the Account was last modified _(in ISO 8601 format)_."""

    email_address: Optional[str] = FieldInfo(alias="emailAddress", default=None)
    """Contact email for the Account."""

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """The ID of the user who last modified the Account."""

    name: Optional[str] = None
    """Name of the Account."""

    parent_account_id: Optional[str] = FieldInfo(alias="parentAccountId", default=None)
    """Parent Account ID, or null if this account does not have a parent."""

    purchase_order_number: Optional[str] = FieldInfo(alias="purchaseOrderNumber", default=None)
    """Purchase Order Number of the Account.

    Optional attribute - allows you to set a purchase order number that comes
    through into invoicing. For example, your financial systems might require this
    as a reference for clearing payments.
    """

    statement_definition_id: Optional[str] = FieldInfo(alias="statementDefinitionId", default=None)
    """
    The UUID of the statement definition used when Bill statements are generated for
    the Account. If no statement definition is specified for the Account, the
    statement definition specified at Organizational level is used.

    Bill statements can be used as informative backing sheets to invoices. Based on
    the usage breakdown defined in the statement definition, generated statements
    give a breakdown of usage charges on Account Bills, which helps customers better
    understand usage charges incurred over the billing period.

    See
    [Working with Bill Statements](https://www.m3ter.com/docs/guides/running-viewing-and-managing-bills/working-with-bill-statements)
    in the m3ter documentation for more details.
    """
